function [preds, targs, llh, subsP, all_lhs]=evalprobitem(data, prec, maxdiff, maxcnt, subsets, folds, combpred)
% function [preds, targs, llh, subsP, all_lhs]=evalprobitem(data, prec, maxdiff, maxcnt, subsets, folds, combpred)
% evaluates linear probit em predictions. 
% IN
% data      - labeled training data
% prec      - precision of Gaussian used to get the probit link
% maxdiff   - maximal eps tolerated in successive EM iterations
% maxcnt    - maximal number of EM iterations
% subsets   - cell array of all subsets to be evaluated.
% folds     - number of folds in predictions.
% combpred  - optional boolean which controls whether we combine
%             predictions of individual subsets. Default is "true".
% OUT
% preds     - predicted probabilities 
%             depending on combpred this has either two columns. 
%             For combpred==1, column 1 are the probabilities for class 0 in a zero 
%             one classification. Otherwise this is a three dimensional matrix with 
%             the third dimension enumerating predicted probabilities over subsets. 
% targs     - true targets
% llh       - vector of log likelihoods of all subsets
% subsP     - llh based subset probabilities.
% all_lhs   - fold based log likelihoods
%
% (C) P. Sykacek 2005

if nargin < 7
    combpred=1;
end
% convert to zero one coding
if (min(data(:,size(data,2)))~=0)
    data(:,size(data,2))=data(:,size(data,2))-min(data(:,size(data,2)));
end
%
preds=[];
targs=[];
all_lhs=[];
for fold=1:folds
    %disp(fold);
    if folds > 1
        [tran, TrgTr, test, TrgTs] = nfldvdat(data ,folds,fold,-1);
        targs=[targs; test(:,size(test,2))];
    else
        tran=data;
        test=[];
    end
    trtargs=tran(:, size(tran,2));
    nsubs=length(subsets);
    llh=zeros(nsubs,1);
    foldpred=zeros(size(test,1), 1, nsubs); 
    for subn=1:nsubs
        in=tran(:,subsets{subn});
        in=[ones(size(in, 1), 1), in];
        % evaluate model
        [w,mz,llh(subn)]=probitem(in, trtargs, prec, maxdiff, maxcnt);
        % predict 
        if ~isempty(test)
            in=test(:,subsets{subn});
            in=[ones(size(test, 1), 1), in];
            % we predict probabilities for class 1 (in a zero one coding)
            [P]=probitempred(w, prec, in);
            foldpred(:,1,subn)=1-P;
        end
    end
    all_lhs=[all_lhs, llh];
    if ~isempty(test)
        preds=[preds; foldpred];
    end
end
llh=sum(all_lhs,2)/folds;
[subsP]=bprb_lgevids2prob(all_lhs', 'a');
if folds > 1
    if combpred
        % get rid of third dimension
        preds=squeeze(preds);
        preds=preds.*subsP(ones(size(preds,1),1),:);
        preds=sum(preds,2);
        preds=[preds, 1-preds];
    else
        preds(:,2,:)=1-preds(:,1,:);
    end
end