function [InTr, TrgTr, InTs, TrgTs] = nfldvdat(X,nfold,step,oneofc)
% function [InTr, TrgTr, InTs, TrgTs] = nfldvdat(X,nfold,step,oneofc)
% Calculates training and test data for n-fold cross validation from the data matrix X.
% X is expected to contain the target data in the last column. The classes are labeled
% with integer numbers starting at 0.
% The target matrices contain the target as 1 of c coding scheme. 
% For c classes the target matrix has c rows with all entries but one set to 0.
% nfold contains mumber of splits in the data; step contains the current step
% e.g. nfold = 10 step = 1..10 are valid.
% InTr and TrgTr contain 100/(nfold-1) % of data the rest is test data.
% oneofc (1 or 0) controls whether a seperate one of c target coding scheme matrix
% (TrgTr and TrgTs) is generated. For R. Neals Bayesian inference we need 
% the target data as last column in the InTr and InTs matrices.

InTr=[]; TrgTr=[]; InTs=[]; TrgTs=[];

% secure that labels start with 0.
if min(X(:,size(X,2)))>0
    X(:,size(X,2))=X(:,size(X,2))-1;
end
NrClss=max(X(:,size(X,2)))+1	% Nr of classes only necessary for one of c target coding.
XRows=size(X,1);

TsStart = round(XRows * (step-1) / nfold)+1;
TsEnd = round(XRows * step / nfold);

NrTst=TsEnd-TsStart+1;

NrTr=XRows-NrTst;

%no problem if second value after colon is less than first -- empty vector !!
TrRows=[1:TsStart-1, TsEnd+1:XRows];		% vector of row indices for training data
TsRows=[TsStart : TsEnd];			% vector of row indices for test data
if oneofc==1
  % one of c target coding as used for nn toolbox
  InTr=X(TrRows,1:size(X,2)-1);			% training data -- input
  InTs=X(TsRows,1:size(X,2)-1);			% test data -- input
  
  TrgTr=zeros(NrTr, NrClss);			% fill target matrices with zeros
  TrgTs=zeros(NrTst, NrClss);
  
  for i=1:NrTr
    TrgTr(i, X(TrRows(i),size(X,2))+1)=1;
  end
  for i=1:NrTst
    TrgTs(i, X(TsRows(i),size(X,2))+1)=1;
  end
else
  % coding used for R.Neals Bayesian inference but reshuffling 
  % of targets not necessary
  InTr=X(TrRows,:);			% training data -- input and targets
  InTs=X(TsRows,:);			% test data -- input and targets
end


