function [padj]=pvaladjust(porig, mthd, ncomp)
% function [padj]=pvaladjust(porig, mthd, ncomp)
% adjusts individual p values for multiple testing.
% IN
% porig   : pvalues of individual hypotheses.
% methd   : string for selecting the method for adjusting 
%           the pi values for multiple testing.
%           default: 'bonferroni'
%           other options: 
%           'ds'     - for Dun Sidak
%           'holm'   - Holm's method
%           'sh'     - Simes-Hochberg method
%           >>> not yet implemented 'hommel' - Hommels method
%           'fdr'    - Benjamini Hochbergs false discovery rate
%           'none'   - no adjusting - makes sure all p-values are smaller 1
%                      and larger 0.
% ncomp    : number of comparisons to adjust for. default is
%            prod(size(porig))
%
% (C) P. Sykacek 2007 <peter@sykacek.net>
% v1.1:  Allows for generic dimensions in porig which is maintained after
%        adjustment.

orig_shape=size(porig);
porig=porig(:);
%padj=porig(:);
if nargin < 2
    methd='bonferroni';
end
if nargin < 3 
    ncomp=length(porig);
end
if strcmp(mthd, 'bonferroni')
    padj=max(0,min(ncomp*porig, 1));
elseif strcmp(mthd, 'ds')
    padj=max(0,min(1-(1-porig).^ncomp,1));
elseif strcmp(mthd, 'holm')
    % sort pvalues
    [porig, idx]=sort(porig);
    % adjust the values such that the comparison with thresholds match
    fakt=ncomp-[0:length(porig)-1]';
    padj=max(0,min(porig.*fakt, 1)); 
    % change adjusted values such that we get for any pvalue the selection
    % that were obtained from holm's rules.
    mxval=padj(1);
    for n=2:length(padj)
        if mxval > padj(n)
            padj(n)=mxval;
        else
            mxval=padj(n);
        end
    end
    padj(idx,:)=padj;
elseif strcmp(mthd, 'sh')
    % sort pvalues
    [porig, idx]=sort(porig);
    % adjust the values such that the comparison with thresholds match
    fakt=ncomp-[0:length(porig)-1]';
    padj=max(0,min(porig.*fakt, 1));
    % change adjusted pvalues such that the comparisons with any threshold match
    % the ones we got from simes hochberg rule.
    minval=padj(length(padj));
    for n=length(padj)-1:-1:1
        if minval < padj(n)
            padj(n)=minval;
        else
            minval=padj(n);
        end
    end
    padj(idx,:)=padj;
elseif strcmp(mthd, 'fdr')    
    % sort pvalues
    [porig, idx]=sort(porig);
    % adjust the values such that thresholding controls the fdr.
    fakt=ncomp./[1:length(porig)]';
    padj=max(0,min(porig.*fakt, 1));
    % change adjusted pvalues such that the comparisons with any threshold match
    % the ones we got from simes hochberg rule.
    minval=padj(length(padj));
    for n=length(padj)-1:-1:1
        if minval < padj(n)
            padj(n)=minval;
        else
            minval=padj(n);
        end
    end
    padj(idx,:)=padj;
elseif strcmp(mthd, 'none')
    padj=max(0,min(porig, 1));
end 
padj=reshape(padj, orig_shape);