function [datacols, seldx]=subsamplegeneids(data, combmode, selspec)
% function [datacols, seldx]=subsamplegeneids(data, combmode, selspec)
% selects random subsets of gene ids from a microarray experiment.
% IN
% data:     log ratios or log amplitude of gene measuerments an 
%           [nsamples x ngenes] matrix of values. Note that in both cases
%           we assume that rows represent replicates of the respective
%           quantities under investigation. If we, wish to subselect based
%           on log ratios, each row represents a repliacte of the
%           respective log ratio under consideration. The same holds for
%           amplitudes.  
% combmode: a function handle applied to data to reduce the matrix to a
%           [1 x ngenes] vector of summary statistics for random selection,
%           e.g. @sum.
% selspec:  a [nquant x 1] vector or [nquant x 2] matrix of subsampling specifications.
%           selspec(:,1) denotes the quantils in the input data. 
%           These can either be integer numbers, in which case 
%           we have max(selspec(:,1)) == ngenes or quantils, in which case
%           we have max(selspec(:,1)) == 1. Note
%           that both quantities refer to upper bounds of selection slices.
%           If available, selspec(:,2) denotes the number
%           of genes randomly selected from the respective quantil,
%           otherwise each quantil specified in selspec(:,1) is split
%           evenly in two random subsets.
% OUT       
% datacols: a [1 x sum(selspec(:,2))] vector of columns to be selected from
%           data for further analysis.
% seldx:    a [1 x sum(selspec(:,2))] vector of intermediate indices (for
%           debugging).
[sdata, idx]=sort(combmode(data));
havequant=max(selspec(:,1))==1;
ngenes=size(data, 2);
if havequant % we need convert quantil values to sample numbers
    for n=1:size(selspec, 1)
        selspec(n,1) = max(1,min(round(selspec(n,1)*ngenes), ngenes));
    end
end
havesplit=size(selspec, 2)==1;
if havesplit
    nquant=size(selspec, 1);
    selspec=[selspec, zeros(nquant, 1)];
    % selspec(:,1) is in any case here given as sample numbers!
    startno=0;
    for n=1:nquant
        selspec(n,2)=floor(0.5*(selspec(n, 1)-startno));
        startno=selspec(n, 1);
    end
end
%selspec
% now generate the gene subset to be extracted.
seldx=[];
startdx=0;
for n=1:size(selspec, 1)
    curdx=startdx+randperm(selspec(n,1)-startdx);
    seldx=[seldx, curdx(1:selspec(n,2))];
    startdx=selspec(n,1);
end
datacols=idx(seldx);
        
    
    